@@ -1,4 +1,4 @@
-; ACL2 Version 6.4 -- A Computational Logic for Applicative Common Lisp
+; ACL2 Version 6.5 -- A Computational Logic for Applicative Common Lisp
 ; Copyright (C) 2014, Regents of the University of Texas
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
@@ -902,6 +902,32 @@ notation causes an error and (b) the use of ,. is not permitted."
                (backquote (car l))
                (backquote-lst (cdr l))))))
 
+(defvar *read-object-comma-count* nil)
+
+(defun acl2-comma-reader (stream)
+  (when *read-object-comma-count*
+    (incf *read-object-comma-count*))
+  (when (< *backquote-counter* 0)
+    (let* ((pathname (and (typep stream 'file-stream)
+                          (pathname stream)))
+           (posn (and pathname
+                      (file-position stream))))
+      (clear-input stream)
+      (cond
+       (posn
+        (error "Illegal comma encountered by READ: file ~a, position ~s."
+               pathname posn))
+       (*read-object-comma-count*
+        (error
+         "Illegal comma: ~:r comma processed while reading top-level form."
+         *read-object-comma-count*))
+       (t (error "Illegal comma encountered by READ.")))))
+  (case (peek-char nil stream t nil t)
+    (#\@ (read-char stream t nil t)
+     (list *comma-atsign* (read stream t nil t)))
+    (#\. (error ",. not allowed in ACL2 backquote forms."))
+    (otherwise (list *comma* (read stream t nil t)))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                            SUPPORT FOR ACL2 CHARACTER READER
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -1339,6 +1365,26 @@ notation causes an error and (b) the use of ,. is not permitted."
     (and (fboundp fn)
          (funcall fn string))))
 
+#+sbcl
+(defmacro define-our-sbcl-putenv ()
+
+; Jared Davis found sb-posix::putenv and the necessary require form (below) for
+; using it.  We aren't finding much documentation about this function on the
+; web, but we also aren't particularly relying on any properties of it.  Our
+; function defined below is a wrapper (if the REQUIRE form succeeds without
+; error), for use in the raw Lisp definition of setenv$.
+
+; But we don't define our-sbcl-putenv here -- instead we define the present
+; macro to be called after starting up ACL2.  The reason is that during the
+; build, environment variable SBCL_HOME might not yet be set; we set it in our
+; function, save-acl2-in-sbcl-aux, so that it's set when ACL2 starts up.
+
+  '(ignore-errors
+    (require :sb-posix)
+    (defun our-sbcl-putenv (var value)
+      (funcall (intern "PUTENV" "SB-POSIX") ; avoid sb-posix package prefix
+               (concatenate 'string var "=" value)))))
+
 (defun get-os ()
 
 ; The following are in priority order.
@@ -1530,7 +1576,7 @@ notation causes an error and (b) the use of ,. is not permitted."
 
 (defun ser-cons-reader-macro (stream subchar arg)
   (declare (ignorable subchar arg))
-  ;; This is the reader macro for #z.  When it is called the #z part has
+  ;; This is the reader macro for #Y.  When it is called the #Z part has
   ;; already been read, so we just want to read the serialized object.
   (ser-decode-from-stream nil :never stream))
 
@@ -1613,3 +1659,4 @@ notation causes an error and (b) the use of ,. is not permitted."
   `(defparameter ,@r)
   #+Clozure
   `(ccl::defstaticvar ,@r))
+
