@@ -1,4 +1,4 @@
-; ACL2 Version 6.4 -- A Computational Logic for Applicative Common Lisp
+; ACL2 Version 6.5 -- A Computational Logic for Applicative Common Lisp
 ; Copyright (C) 2014, Regents of the University of Texas
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
@@ -701,7 +701,7 @@ implementations.")
  + WARNING: This is NOT an ACL2 release; it is svn revision ~a.     +
  + The authors of ACL2 consider svn distributions to be experimental; +
  + they may be incomplete, fragile, and unable to pass our own        +
- + regression.  Bug reports should include the following line:        +
+ + regression tests.  Bug reports should include the following line:  +
  +   ACL2 svn revision ~a; community books svn revision ~a        +
  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 "
@@ -1573,9 +1573,13 @@ implementations.")
         str
 
 ; In order to profile, Nikodemus Siivola has told us that we "need to set
-; SBCL_HOME to the location of the contribs".  Using apropos we found the
-; function SB-INT:SBCL-HOMEDIR-PATHNAME that returns the right directory in
-; SBCL Version 1.0.23.  So we'll use that.
+; SBCL_HOME to the location of the contribs".  We used contrib/ through SBCL
+; 1.1.11.  But we noticed in SBCL 1.1.14 that the suitable contrib/ directory
+; (at least for requiring sb-sprof) is obj/sbcl-home/contrib/, an impression
+; that seemed to be confirmed via email from fahree@gmail.com on 1/13/2014.
+; Since directory obj/sbcl-home/ doesn't exist in (our installation of) SBCL
+; 1.1.11, we simply look for that first.  But we noticed that it doesn't work
+; to include the trailing "contrib/" when using obj/sbcl-home/.
 
         ("~a~%"
          (let ((contrib-dir
@@ -1586,6 +1590,13 @@ implementations.")
                         (let* ((core-dir
                                 (pathname-directory
                                  sb-ext::*core-pathname*))
+                               (contrib-dir-pathname-new ; see comment above
+                                (and (equal (car (last core-dir))
+                                            "output")
+                                     (make-pathname
+                                      :directory
+                                      (append (butlast core-dir 1)
+                                              (list "obj/sbcl-home")))))
                                (contrib-dir-pathname
                                 (and (equal (car (last core-dir))
                                             "output")
@@ -1593,9 +1604,13 @@ implementations.")
                                       :directory
                                       (append (butlast core-dir 1)
                                               (list "contrib"))))))
-                          (and (probe-file contrib-dir-pathname)
-                               (setq *sbcl-contrib-dir*
-                                     (namestring contrib-dir-pathname)))))))))
+                          (cond ((probe-file contrib-dir-pathname-new)
+                                 (setq *sbcl-contrib-dir*
+                                       (namestring contrib-dir-pathname-new)))
+                                ((probe-file contrib-dir-pathname)
+                                 (setq *sbcl-contrib-dir*
+                                       (namestring contrib-dir-pathname)))
+                                (t nil))))))))
            (if contrib-dir
                (format nil
                        "export SBCL_HOME=~s"
