@@ -1,4 +1,4 @@
-; ACL2 Version 6.4 -- A Computational Logic for Applicative Common Lisp
+; ACL2 Version 6.5 -- A Computational Logic for Applicative Common Lisp
 ; Copyright (C) 2014, Regents of the University of Texas
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
@@ -250,6 +250,7 @@
         EXTEND-32-BIT-INTEGER-STACK
         EXTEND-T-STACK
         EXTEND-WORLD EXTRA-INFO F-GET-GLOBAL F-PUT-GLOBAL
+        FAST-ALIST-CLEAN FAST-ALIST-CLEAN! FAST-ALIST-FORK FAST-ALIST-FORK!
         FAST-ALIST-FREE-ON-EXIT FERTILIZE FC-REPORT
         FGETPROP FIFTH FILE-CLOCK FILE-CLOCK-P
         FILE-CLOCK-P-FORWARD-TO-INTEGERP FINALIZE-EVENT-USER
@@ -268,6 +269,7 @@
         GUARD GUARD-OBLIGATION HARD-ERROR
         HAS-PROPSP HAS-PROPSP1 HEADER HELP HIDE
         HONS-ASSOC-EQUAL HONS-COPY-PERSISTENT
+        HONS-SHRINK-ALIST HONS-SHRINK-ALIST!
         I-AM-HERE ID IDATES IDENTITY
         IF IF* IFF IFF-IMPLIES-EQUAL-IMPLIES-1
         IFF-IMPLIES-EQUAL-IMPLIES-2
