@@ -7,15 +7,25 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Jared Davis <jared@centtech.com>
 */
@@ -142,6 +152,24 @@ function key_title(key)
        : ("XDOC &mdash; " + key);
 }
 
+
+
+function apply_suborder(subkeys, keys) {
+    var ret = [];
+    for(var i in subkeys) {
+	ret.push(subkeys[i]);
+    }
+    for(var i in keys) {
+        var k = keys[i];
+	var idx = ret.indexOf(k);
+	if (idx == -1) { // new key, add it
+	    ret.push(k);
+	}
+	// else already have it.
+    }
+    return ret;
+}
+
 function key_sorted_children(key) { // Returns a nicely sorted array of child_keys
     var children = topic_child_keys(key);
 
@@ -157,6 +185,12 @@ function key_sorted_children(key) { // Returns a nicely sorted array of child_ke
     for(var i in tmp) {
         ret.push(tmp[i].key);
     }
+
+    var suborder = topic_suborder(key);
+    if (suborder.length > 0) {
+	return apply_suborder(suborder, ret);
+    }
+
     return ret;
 }
 
@@ -831,6 +865,12 @@ $(document).ready(function()
 });
 
 
+function jump_render(datum) {
+    var key = datum["value"];
+    return "<p><b class=\"sf\">" + topic_name(key) + "</b> &mdash; " +
+	topic_short_plaintext(key) + "<br/><tt>" + key + "</tt></p>";
+}
+
 function jump_init() {
 
     var ta_data = [];
@@ -840,20 +880,17 @@ function jump_init() {
         var tokens = [];
         tokens.push(topic_rawname(key));
         var entry = {"value": key,
-                     "nicename": topic_name(key),
-                     "short": render_text(topic_short(key)),
+		     "nicename": topic_name(key),
                      "tokens": tokens};
         ta_data.push(entry);
     }
 
     $("#jump").typeahead([{
-            name: "topics",
-            local: ta_data,
-            limit: 6,
+        name: "topics",
+	local: ta_data,
+	limit: 6,
 	autoselect: 'first',
-            template: "<p><b class=\"sf\">{{{nicename}}}</b> &mdash; {{{short}}}<br/>"
-                     + "<tt>{{value}}</tt></p>",
-            engine: Hogan
+	template: jump_render
     }]);
 
     $("#jump").bind('typeahead:selected', jump_go);
